/////////////////////////////////////////////////////////////////////////////
///
/// @file cmgraphs.cpp
///
/// Conley-Morse graphs - the main program.
/// This program computes a set of Conley-Morse graphs
/// for a given discrete dynamical system
/// for a prescribed rectangular range of parameters,
/// and establishes the continuation relation
/// between adjacent computed graphs.
///
/// @author Pawel Pilarczyk
///
///
/// @mainpage The Conley-Morse Graphs Software
///
/// \section intro Introduction
///
/// This website contains the documentation of software designed and
/// programmed by <a href="https://www.pawelpilarczyk.com/">Pawel Pilarczyk</a>
/// and accompanies the paper
/// <em>Databases for the Global Dynamics of Multiparameter Systems</em>
/// by Z. Arai, W. Kalies, H. Kokubu, K. Mischaikow, H. Oka,
/// and P. Pilarczyk. Please, see the
/// <a href="https://www.pawelpilarczyk.com/database/">website of the project</a>
/// and refer to the <a href="https://www.pawelpilarczyk.com/cmgraphs/">page
/// devoted to the software</a> for more information.
///
/// \section license License
///
/// In order to make this software freely available for wide audience,
/// it is provided here under the terms of the
/// <a href="http://www.gnu.org/copyleft/gpl.html">GNU General
/// Public License</a>, and this documentation is provided under the terms
/// of the <a href="http://www.gnu.org/copyleft/fdl.html">GNU Free
/// Documentation License</a>.
///
/// \section browsing Browsing
///
/// Although a lot of effort has been put into making this software
/// as transparent and clear as possible,
/// and to make this documentation complete,
/// I am sure that there is a lot of important information
/// which should have been included here but was omitted.
/// Therefore, this documentation was generated in such
/// a way that it contains the entire source code
/// which can be browsed and verified in case of any doubt.
///
/// Have a nice browsing!
///
/// Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2014 by Pawel Pilarczyk.
//
// This file is part of my research software package. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <https://www.gnu.org/licenses/>.

// Started in November 2005. Last revision: July 7, 2014.


// include some standard C++ header files
#include <exception>

// include selected header files from the CHomP library
#include "chomp/system/config.h"
#include "chomp/system/textfile.h"
#include "chomp/system/timeused.h"
#include "chomp/system/arg.h"
#include "chomp/multiwork/mw.h"

// include local header files
#include "config.h"
#include "typedefs.h"
#include "typedyns.h"
#include "worker.h"
#include "coord.h"


// --------------------------------------------------
// -------------------- OVERTURE --------------------
// --------------------------------------------------

/// The title of the program which is displayed every time
/// the program is launcued.
const char *title = "\
This is a Conley-Morse graphs computation program.\n\
Ver. 0.09, July 15, 2014. Copyright (C) 2005-2014 by Pawel Pilarczyk.\n\
This is free software. No warranty. Consult 'license.txt' for details.";

/// The help information about the program which is displayed
/// if the program is launched without command-line arguments
/// or with incorrect arguments.
const char *helpinfo = "\
This is the front-end program for the computation of Conley-Morse graphs\n\
for a selected range of parameters for a given discrete dynamical system,\n\
as configured in the file 'config.h' of the source code. Please, see the\n\
documentation of the program and the source code for more information.\n\
Command line arguments (either '-w', or '-c' and '-f' must be specified):\n\
-w computer:port - run as a worker and connect to the given coordinator,\n\
-c [port] - run as a coordinator and listen to workers at the given port;\n\
\tif port=0 (default) then runs the computations locally only,\n\
-f prefix - file name prefix for saving the final continuation data,\n\
-i filename - intermediate results file (new results will be appended),\n\
-g filename - file name to save the text code for Conley-Morse graphs,\n\
-m prefix - file name prefix for text code of Conley-Morse graphs,\n\
-s prefix - file name prefix for shared info on Morse decompositions,\n\
-p prefix - file name prefix for PNG pictures of Morse decompositions,\n\
-q prefix - file name prefix for Morse sets as lists of cubes,\n\
-o prefix - file name prefix for post-processing Morse sets information,\n\
-t filename - file name with optimization information (read & append),\n\
-u prefix - file name prefix for ODE integration optimization information,\n\
-z prefix - prefix for temporary files with Morse decompositions cache,\n\
-n num - the number of parameter space patches for matching;\n\
\tvalue <0 sets the distance between patch corners in each direciton,\n\
--full - use the full range of the phase space to plot Morse sets,\n\
--log filename - file name to save all the text output of the program to,\n\
--quiet - suppress any output written to the terminal,\n\
--debug - display a lot of additional information (for debugging etc.),\n\
--help - make the program show this brief help information and exit.\n\
For more information ask the author at http://www.pawelpilarczyk.com/.";


// --------------------------------------------------
// ---------------------- MAIN ----------------------
// --------------------------------------------------

/// The main procedure of the program.
/// Returns: 0 = Ok, -1 = Error, 1 = Help displayed, 2 = Wrong arguments.
int main (int argc, char *argv [])
{
	using namespace chomp::homology;

	// prepare user-configurable data
	char *workAddress = 0;
	char *interFileName = 0;
	char *mapOptFileName = 0;
	char *mapOptPrefix = 0;
	char *graphsFileName = 0;
	char *graphsPrefix = 0;
	char *finalPrefix = 0;
	char *sharePrefix = 0;
	char *phaseSpacePrefix = 0;
	char *morseDecPrefix = 0;
	char *cubesPrefix = 0;
	char *procPrefix = 0;
	int coordPort = -1;
	int skipIndices = 0;
	bool fullPhaseSpace = false;
	int nPatches = 0;
	const int maxComments = 16;
	int nComments = 0;
	char *comments [maxComments];

	// interprete the command-line arguments
	arguments a;
	arg (a, "c", coordPort, 0);
	arg (a, "w", workAddress);
	arg (a, "i", interFileName);
	arg (a, "t", mapOptFileName);
	arg (a, "u", mapOptPrefix);
	arg (a, "g", graphsFileName);
	arg (a, "m", graphsPrefix);
	arg (a, "f", finalPrefix);
	arg (a, "s", sharePrefix);
	arg (a, "q", cubesPrefix);
	arg (a, "o", procPrefix);
	arg (a, "p", phaseSpacePrefix);
	arg (a, "z", morseDecPrefix);
	arg (a, "n", nPatches);
	arg (a, "-comment", comments, nComments, maxComments);
	arg (a, "-skip-indices", skipIndices, 1);
	argswitch (a, "-full", fullPhaseSpace, true);
	arghelp (a);

	argstreamprepare (a);
	int argresult = a. analyze (argc, argv);
	argstreamset ();

	// if the final results prefix is specified but no port for the
	// coordinator was specified then set it to 0 (local work)
	if (finalPrefix && (coordPort < 0))
		coordPort = 0;

	// if no worker/coordinator selected then show help information
	if (!finalPrefix && !workAddress)
		argresult = 1;

	// show the program's main title
	if (argresult >= 0)
		sout << title << '\n';

	// if something was incorrect, show an additional message and exit
	if (argresult < 0)
	{
		sout << "Call with '--help' for help.\n";
		return 2;
	}

	// if help requested, show help information
	if (argresult > 0)
	{
		sout << helpinfo << '\n';
		return 1;
	}

	// try running the main function and catch an error message if thrown
	try
	{
		// set an appropriate program time message
		program_time = "Aborted after:";
		program_time = 1;

		// run as a worker if requested to
		if (workAddress)
		{
			Worker w (false);
			w. Port (0);
			w. Add (workAddress);
			w. TimeOut (600);
			sout << "Running as a worker...\n";
			int result = w. Work ();
			if (result == chomp::multiwork::mwOk)
				sout << "Work completed successfully.\n";
			else
				sout << "Could not work - probably "
					"an error occurred.\n";
		}

		// run as a coordinator otherwise
		else
		{
			Worker w (true);
			Coordinator c (interFileName,
				mapOptFileName, mapOptPrefix,
				graphsFileName, graphsPrefix,
				finalPrefix, sharePrefix,
				phaseSpacePrefix, cubesPrefix,
				morseDecPrefix, procPrefix,
				fullPhaseSpace, skipIndices, nPatches);
			c. Port (coordPort);
			sout << "Running as a coordinator...\n";
			int result = c. Coordinate (coordPort ? 0 : &w);
			if (result == chomp::multiwork::mwOk)
				sout << "The task completed successfully.\n";
			else
				sout << "Could not coordinate - probably "
					"an error occurred.\n";
		}

		// set an appropriate program time message
		program_time = "Total time used:";

		// finalize
		return 0;
	}
	catch (const char *msg)
	{
		sout << "ERROR: " << msg << '\n';
		return -1;
	}
	catch (const std::exception &e)
	{
		sout << "ERROR: " << e. what () << '\n';
		return -1;
	}
	catch (...)
	{
		sout << "ABORT: An unknown error occurred.\n";
		return -1;
	}
} /* main */

