/////////////////////////////////////////////////////////////////////////////
///
/// @file config.h
///
/// Choice of configuration settings.
/// This is a file that sets a few special configuration options,
/// and includes an appropriate configuration file
/// for the program "cmgraphs.cpp".
/// The included configuration file defines the map which is analyzed,
/// the ranges of parameters, and the phase space information.
///
/// @namespace custom
///
/// Customizable settings that are supposed to be modified and/or chosen
/// by the user of the software. This namespace contains various constants,
/// classes and functions which can be modified in order to customize
/// the programs for the computation of Conley-Morse graphs
/// and the continuation diagrams.
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2022 by Pawel Pilarczyk.
//
// This file is part of my research software package. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <https://www.gnu.org/licenses/>.

// Started on February 9, 2008. Last revision: July 18, 2022.


// --------------------------------------------------
// --------- SPECIAL CONFIGURATION OPTIONS ----------
// --------------------------------------------------

// Use single parameter points instead of parameter intervals.
// Comment this setting out for the final computations.
//#define CONFIG_PARAMPOINTS

// Do not send individual parameter boxes for precomputing
// the Conley indices.
//#define CONFIG_NOPREPROCESSING

// Do preprocessing only and skip the matching part.
//#define CONFIG_ONLYPREPROCESSING

// Wait with Phase 2 until Phase 1 (preprocessing) has been completed.
#define CONFIG_SEPARATEPHASES

// Use the procedure for computing the Morse decomposition
// instead of computing the invariant part of the set,
// and also don't determine connections between Morse sets.
// In this way, the connections between Morse sets are lost,
// but the invariant part to subdivide may be substantially smaller.
// Comment this setting out for the final computations
// to get full information.
//#define CONFIG_NOCONNECTIONS

// Use the default bounding box for ODE integration in order to prevent
// iterating trajectories far beyond the phase space of interest,
// and, in particular, to avoid running out of the scope of integer
// coordinates of boxes when determining the images of boxes.
// WARNING: This technique may lead to the construction of combinatorial
// cubical multivalued maps that are not acyclic, that is, do not satisfy
// the necessary assupmtions for the computation of the homomorphism
// induced in homology (for the Conley index map).
// NOTE: If using the approximate integration of the ODE, this technique
// works in a different way, by fading the vector field outside the box,
// so it is safe to use it in that setting.
//#define CONFIG_ODEBOUNDINGBOX

// Make a correction to the isolating neiborhood after subdivision.
// This is necessary for integrating ODEs using the time-t map
// with variable t, because an isolating neighborhood for one t
// need not be an isolating neighborhood for another t, so it is only taken
// as a hint, and those cubes from the exit set whose images intersect
// the isolating neighborhood are added to the isolating neighborhood,
// like in an algorithm for the construction of an index pair.
//#define CONFIG_ODEVART

// Add cubes to the image of a combinatorial cubical multivalued map
// only if their intersection with the interval box that covers the image
// is equal or higher to the provided fraction.
// Set the fraction to 0 to add whenever they intersect (old behavior),
// set the fraction to a maximum of 1 (to add those only which are contained).
//#define CONFIG_IFRACTION 0.05

// Enhance the invariant part the given number of times
// before proceeding to the next subdivision level.
//#define CONFIG_ENHANCEINV 1

// Define the X coordinate for plotting the Morse sets in the PNG files.
//#define PLOT_X_COORD 1

// Define the Y coordinate for plotting the Morse sets in the PNG files.
//#define PLOT_Y_COORD 3


// --------------------------------------------------
// --------------- CONFIGURATION FILE ---------------
// --------------------------------------------------

//#include "c_differ.h"
//#include "c_duffing_a.h"
//#include "c_globclog.h"
//#include "c_harvest.h"
//#include "c_henon.h"
//#include "c_inf_t.h"
//#include "c_lorenz.h"
//#include "c_lorenz_a.h"
//#include "c_lorenz_t.h"
#include "c_neuron09.h"
//#include "c_neuron12.h"
//#include "c_neuron25.h"
//#include "c_pop2demo.h"
//#include "c_pop2p.h"
//#include "c_roessler_a.h"
//#include "c_stdmap.h"
//#include "c_twocirc.h"
//#include "c_twotorus.h"
//#include "c_vpol.h"
//#include "c_vpol1.h"
//#include "c_vpol_a.h"
//#include "c_whales.h"

