/////////////////////////////////////////////////////////////////////////////
///
/// @file p_globclog.h
///
/// Parameters for the globally coupled logistic maps.
/// This is a configuration file for the program "cmgraphs.cpp"
/// which defines the ranges of parameters to scan
/// and the details of the phase space.
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2014 by Pawel Pilarczyk.
//
// This file is part of my research software package. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <https://www.gnu.org/licenses/>.

// Started on April 10, 2008. Last revision: April 12, 2008.


#ifndef _CMGRAPHS_P_H_
#define _CMGRAPHS_P_H_


namespace custom {

/// Parameters for the globally coupled logistic maps.
namespace globclog {


// --------------------------------------------------
// --------- the phase space configuration ----------
// --------------------------------------------------

/// The dimension of the phase space.
const int spaceDim = 3;

/// Is space wrapping (a.k.a. periodic boundary conditions)
/// in effect in the given directions?
const bool spaceWrapping [spaceDim] = {false, false, false};

/// The initial depth of subdivisions in the phase space.
const int initialDepth = 6;

/// The final depth of subdivisions in the phase space.
const int finalDepth = 7;


// --------------------------------------------------
// ------------ the parameter selection -------------
// --------------------------------------------------

/// The dimension of the parameter space to iterate. Only those parameters
/// which are varying are taken into consideration here.
const int paramDim = 2;

/// The numbers of subintervals in each direction of the parameter space.
/// The type of these numbers must match "parCoord" in "typedefs.h".
const short int paramSubdiv [paramDim] = {128, 128}; //{32000, 32000};

/// The numbers of parameters to subdivide.
const int paramSelect [paramDim] = {0, 1};


// --------------------------------------------------
// ---------- the parameter space settings ----------
// --------------------------------------------------

/// The number of all the parameters, both varying and fixed.
const int paramCount = 2;

/// The left bounds on the parameters.
const double paramLeft [paramCount] = {1.7462857142857142, 0.3251953125};
//{1.75, 0.17}; //{1.4, 0.0};

/// The right bounds on the parameters.
const double paramRight [paramCount] = {1.7462857142857142, 0.3251953125};
//{1.8, 0.22}; //{2.0, 0.4};


// --------------------------------------------------
// -------------- the phase space size --------------
// --------------------------------------------------

/// An imitation of an array which returns the offset of the rectangular area
/// in the phase space which contains the invariant sets of interest.
struct SpaceOffsetType
{
	/// Returns the space offset in the requested direction.
	double operator [] (int) const
	{
		return -1.0; //-1.7;
	}
}; /* struct SpaceOffsetType */

/// An imitation of an array which returns the offset of the rectangular area
/// in the phase space which contains the invariant sets of interest.
const SpaceOffsetType spaceOffset = SpaceOffsetType ();

/// An imitation of an array which returns the width of the rectangular area
/// in the phase space which contains the invariant sets of interest.
struct SpaceWidthType
{
	/// Returns the space width in the requested direction.
	double operator [] (int) const
	{
		return 2.1; //2.8;
	}
}; /* struct SpaceWidthType */

/// An imitation of an array which returns the width of the rectangular area
/// in the phase space which contains the invariant sets of interest.
const SpaceWidthType spaceWidth = SpaceWidthType ();


// --------------------------------------------------
// ------------------ refinements -------------------
// --------------------------------------------------

/// The number of refinements that should be done if a Morse set
/// with the trivial index is encountered or if the image of a Morse set
/// sticks out of the rectangular region to which the computations
/// are restricted. Set to zero for no refinements.
const int refineDepth = 4;

/// The maximal allowed size of a set of cubes in the phase space
/// which can be refined at the initial subdivision level.
/// Set to zero to suppress any refinements.
const int maxRefineSize0 = 120000;

/// The maximal allowed size of a set of cubes in the phase space
/// which can be refined at the subsequent subdivision levels.
/// Set to zero to suppress any refinements.
const int maxRefineSize1 = 50000;


// --------------------------------------------------
// ------------------- map images -------------------
// --------------------------------------------------

/// The maximal allowed diameter of the cubical image of a signle box.
/// This limit is set up in order to prevent a blow-up of the program
/// in the memory because of accumulating huge images of boxes.
const int maxImageDiameter = 15000;

/// The maximal allowed volume of the cubical image of a single box.
/// This limit is set up in order to prevent a blow-up of the program
/// in the memory because of accumulating huge images of boxes.
const int maxImageVolume = 30000;


// --------------------------------------------------
// -------------- max index pair size ---------------
// --------------------------------------------------

/// The maximal allowed size of the index pair. This size is measured
/// as the total number of cubes in the index pair and its forward image.
const int maxIndexPairSize = 2000000;


// --------------------------------------------------
// --------------- joining Morse sets ---------------
// --------------------------------------------------

/// The maximal number of cubes in a trivial Morse set for which an attempt
/// is made to join this set with another near-by Morse set.
/// Set to zero to suppress joining.
const int maxJoinSize = 0;

/// The maximal size of a connecting orbit between two Morse sets
/// which can be considered for joining.
const int maxJoinConnection = 1000;

/// The maximal allowed distance between two Morse sets which can be
/// considered for joining. Note: It doesn't work with space wrapping.
const int maxJoinDistance = 100;


// --------------------------------------------------
// ----------------- other settings -----------------
// --------------------------------------------------

/// Should the ordering between the Morse sets be taken into consideration
/// while determining whether two Morse decompositions
/// computed for adjacent parameter boxes are in the same class?
const bool compareMorseOrdering = true;

/// Ignoring the isolation problem while matching Morse decompositions.
/// If this constant is set to "true" then Morse sets are matched
/// with each other indepent of whether the Conley index could be computed
/// for them or not. Otherwise, 'correct' and 'wrong' Morse sets
/// are matched with their counterparts in the same category only.
const bool ignoreIsolationForContinuation = true;

/// Ignoring the isolation problem while computing the Conley index.
/// If the constant is set to "true" then the index pair constructed on the
/// basis of a combinatorial Morse set with respect to the dynamics
/// restricted to the rectangular area of interest is also restricted
/// to that area by means of projecting any boxes which stick out
/// of the region onto a single layer of boxes around the boundary.
const bool ignoreIsolationForConleyIndex = false;


} // namespace globclog
} // namespace custom


#endif // _CMGRAPHS_P_H_

