/////////////////////////////////////////////////////////////////////////////
///
/// @file p_whales.h
///
/// Parameters for the whales population model.
/// This is a configuration file for the program "cmgraphs.cpp"
/// which defines the ranges of parameters to scan
/// and the details of the phase space.
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2014 by Pawel Pilarczyk.
//
// This file is part of my research software package. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <https://www.gnu.org/licenses/>.

// Started on February 9, 2008. Last revision: September 11, 2011.


#ifndef _CMGRAPHS_P_H_
#define _CMGRAPHS_P_H_


namespace custom {

/// Parameters for sample computations for the 2-dimensional Leslie
/// population model with 2 varying parameters.
namespace whales {


// --------------------------------------------------
// --------- the phase space configuration ----------
// --------------------------------------------------

/// The dimension of the phase space.
const int spaceDim = 2;

/// Is space wrapping (a.k.a. periodic boundary conditions)
/// in effect in the given directions?
const bool spaceWrapping [spaceDim] = {false, false};

/// The initial depth of subdivisions in the phase space.
const int initialDepth = 6;

/// The final depth of subdivisions in the phase space.
const int finalDepth = 11;


// --------------------------------------------------
// ------------ the parameter selection -------------
// --------------------------------------------------

/// The dimension of the parameter space to iterate. Only those parameters
/// which are varying are taken into consideration here.
const int paramDim = 2;

/// The numbers of subintervals in each direction of the parameter space.
/// The type of these numbers must match "parCoord" in "typedefs.h".
const short int paramSubdiv [paramDim] = {100, 100};

/// The numbers of parameters to subdivide.
const int paramSelect [paramDim] = {0, 1};


// --------------------------------------------------
// ---------- the parameter space settings ----------
// --------------------------------------------------

/// The number of all the parameters, both varying and fixed.
/// 0 - survival rate for adults: s_a in [0,1]
/// 1 - survival rate for juveniles: s_j in [0,1]
/// 2 - alpha = e^r > 1 (e.g. e^{4} = approx 54.598)
/// 3 - beta > 0 (e.g. e^{r-1} = 20.086 - phase space scaling factor)
/// 4 - options bits:
///	0x010 for "s_j := 1 - s_a",
///	0x020 for "s_j := s_a",
///	0x100 for "beta := alpha / e".
const int paramCount = 5;

/// The left bounds on the parameters.
const double paramLeft [paramCount] = {0.0, 0.0,
	54.598150033144236, 20.085536923187668, 0x100};

/// The right bounds on the parameters.
const double paramRight [paramCount] = {0.9, 0.9,
	54.598150033144236, 20.085536923187668, 0x100};


// --------------------------------------------------
// -------------- the phase space size --------------
// --------------------------------------------------

/// An array which keeps the offset of the rectangular area
/// in the phase space that contains the invariant sets of interest.
/// NOTE: The map is defined in such a way that all non-positive images
/// are forced to be slightly positive; thus the phase space offset
/// must not be negative.
const double spaceOffset [spaceDim] = {0.0, 0.0};

/// An array which keeps the width of the rectangular area
/// in the phase space that contains the invariant sets of interest.
const double spaceWidth [spaceDim] = {4.0, 4.0};


// --------------------------------------------------
// ------------------ refinements -------------------
// --------------------------------------------------

/// The number of refinements that should be done if a Morse set
/// with the trivial index is encountered or if the image of a Morse set
/// sticks out of the rectangular region to which the computations
/// are restricted. Set to zero for no refinements.
const int refineDepth = 15;

/// The maximal allowed size of a set of cubes in the phase space
/// which can be refined at the initial subdivision level.
/// Set to zero to suppress any refinements.
const int maxRefineSize0 = 10000;

/// The maximal allowed size of a set of cubes in the phase space
/// which can be refined at the subsequent subdivision levels.
/// Set to zero to suppress any refinements.
const int maxRefineSize1 = 5000;


// --------------------------------------------------
// ------------------- map images -------------------
// --------------------------------------------------

/// The maximal allowed diameter of the cubical image of a signle box.
/// This limit is set up in order to prevent a blow-up of the program
/// in the memory because of accumulating huge images of boxes.
const int maxImageDiameter = 10000;

/// The maximal allowed volume of the cubical image of a single box.
/// This limit is set up in order to prevent a blow-up of the program
/// in the memory because of accumulating huge images of boxes.
const int maxImageVolume = 100000;


// --------------------------------------------------
// -------------- max index pair size ---------------
// --------------------------------------------------

/// The maximal allowed size of the index pair. This size is measured
/// as the total number of cubes in the index pair and its forward image.
const int maxIndexPairSize = 10000000;


// --------------------------------------------------
// --------------- joining Morse sets ---------------
// --------------------------------------------------

/// The maximal number of cubes in a trivial Morse set for which an attempt
/// is made to join this set with another near-by Morse set.
/// Set to zero to suppress joining.
const int maxJoinSize = 0;

/// The maximal size of a connecting orbit between two Morse sets
/// which can be considered for joining.
const int maxJoinConnection = 1000;

/// The maximal allowed distance between two Morse sets which can be
/// considered for joining. Note: It doesn't work with space wrapping.
const int maxJoinDistance = 100;


// --------------------------------------------------
// ----------------- other settings -----------------
// --------------------------------------------------

/// Should the ordering between the Morse sets be taken into consideration
/// while determining whether two Morse decompositions
/// computed for adjacent parameter boxes are in the same class?
const bool compareMorseOrdering = true;

/// Ignoring the isolation problem while matching Morse decompositions.
/// If this constant is set to "true" then Morse sets are matched
/// with each other indepent of whether the Conley index could be computed
/// for them or not. Otherwise, 'correct' and 'wrong' Morse sets
/// are matched with their counterparts in the same category only.
const bool ignoreIsolationForContinuation = true;

/// Ignoring the isolation problem while computing the Conley index.
/// If the constant is set to "true" then the index pair constructed on the
/// basis of a combinatorial Morse set with respect to the dynamics
/// restricted to the rectangular area of interest is also restricted
/// to that area by means of projecting all the boxes that stick out
/// of the region onto a single layer of boxes around the boundary.
const bool ignoreIsolationForConleyIndex = false;


} // namespace whales
} // namespace custom


#endif // _CMGRAPHS_P_H_

