/////////////////////////////////////////////////////////////////////////////
///
/// @file procexit.h
///
/// A plug-in for post-processing of Morse decompositions
/// which saves the real exit sets of all the Morse sets.
/// This is a replacement for "procmdec.h".
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2014 by Pawel Pilarczyk.
//
// This file is part of my research software package. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <https://www.gnu.org/licenses/>.

// Started on October 26, 2010. Last revision: October 26, 2010.


#ifndef _CMGRAPHS_PROCEXIT_H_
#define _CMGRAPHS_PROCEXIT_H_

// include some standard C++ header files
#include <sstream>
#include <fstream>
#include <string>

// include selected header files from the CHomP library
#include "chomp/system/textfile.h"
#include "chomp/struct/digraph.h"
#include "chomp/struct/flatmatr.h"
#include "chomp/struct/multitab.h"

// include local header files
#include "config.h"
#include "typedefs.h"
#include "conindex.h"
#include "morsedec.h"
#include "typedyns.h"
#include "utils.h"


namespace custom {

/// Post-processing of Morse decompositions by means of cycle decomposition.
namespace procMorseDecExit {


// --------------------------------------------------
// ------- post-process a Morse decomposition -------
// --------------------------------------------------

/// Post-processes a Morse decomposition by computing a decomposition
/// of each Morse set into cycle sets and analyzing this decomposition.
template <class CubSetType, class MorseDecType, class GraphType,
	class CubMapType>
inline void processMorseDec (const MorseDecType &morseDec,
	const CubSetType &allCubes, const GraphType &g,
	const CubMapType &theCubMap, const CubMapType &,
	const std::string &cubesFilePrefix,
	const std::string &procFilePrefix)
{
	using namespace chomp::homology;

	sout << "Computing and saving exit sets of the Morse sets.\n";

	// analyze each Morse set
	int nSets = morseDec. count ();
	for (int n = 0; n < nSets; ++ n)
	{
		const CubSetType &morseCubes = morseDec [n];
		int_t morseCount = morseCubes. size ();

		// restrict the analysis to the given Morse set
		sbug << "Morse set no. " << n << ": " <<
			morseCount << " cubes. Exit set: ";

		// compute and save the real exit set for the Morse set
		SetOfCubes img;
		for (int i = 0; i < morseCount; ++ i)
			theCubMap (morseCubes [i], &img, 0, 0, false);
		img. remove (morseCubes);
		sbug << img. size () << " cubes.\n";

		// prepare a file name for this Morse set
		std::ostringstream exitFileName;
		exitFileName << cubesFilePrefix << n << "e.cub";

		// save the exit set to a file
		std::ofstream f (exitFileName. str (). c_str ());
		f << "; The exit set for the corresponding Morse set (" <<
			img. size () << " cubes).\n";
		f << img;
		f. close ();
	}

	return;
} /* processMorseDec */


} // namespace procMorseDecExit
} // namespace custom


#endif // _CMGRAPHS_PROCEXIT_H_

