#!/usr/bin/perl

#############################################################################
#
# This script returns computation parameters, either the ones hard-coded
# in the script, or automatically determined (if possible).
#
#############################################################################

# Copyright (C) 1997-2022 by Pawel Pilarczyk.
#
# This file is part of my research software package. This is free software:
# you can redistribute it and/or modify it under the terms of the GNU
# General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; see the file "license.txt". If not,
# please, see <http://www.gnu.org/licenses/>.

# Started on April 13, 2011. Last revision: February 12, 2022.

use Cwd;
use Sys::Hostname;
# using File::HomeDir requires a relatively recent version of Perl (5.12.3+)
#use File::HomeDir;

my @arguments = @ARGV;

my %desc = (
	"user" => "the name of the current user",
	"home" => "the user's home directory",
	"name" => "the name of the computations set, e.g. comp21",
	"number" => "the number of the computations set, e.g. 21",
	"port" => "the coordinator's port number, e.g. 8321",
	"host" => "the coordinator's host base (or current if not found)",
#	"domain" => "the coordinator's domain (or current if not found)",
	"coord" => "the coordinator's computer:port, e.g. conley2:8321",
#	"dirprefix" => "work directory prefix, e.g. /data/",
	"dirbase" => "the base results directory name, e.g. /data/comp21",
	"curdir" => "the current directory from which the script is called",
	"zipdir" => "the directory for storing zipped data files",
	"debug" => "detailed information for debugging this script",

	"datdir" => "the data directory, e.g. /data/comp21.dat",
	"cubdir" => "the directory for cubes, e.g. /data/comp21.cub",
	"sharedir" => "the shared indices directory, e.g. /data/comp21.ind",
	"pngdir" => "the png images directory, e.g. /data/comp21.png",
	"odedir" => "the ODE parameters directory, e.g. /data/comp21.ode",
	"decdir" => "the Morse dec cache directory, e.g. /data/comp21.dec",
	"cmgdir" => "the CMGraphs directory, e.g. /data/comp21.cmg",
	"rundir" => "the directory for a copy of running data (ext. run)",
	"postprocdir" => "the directory for postprocessing (ext. pos)",
	"scheduler" => "the name of the queue scheduler (PBS, SLURM, none)",
);

unless (scalar @arguments)
{
	print "This script called with one of the following parameters\n";
	print "returns the following information about the computations:\n";
	foreach my $param (sort keys %desc)
	{
		print $param, " - ", $desc{$param}, "\n";
	}
	exit;
}

our $debug = 0;
if ($arguments[0] eq "debug")
{
	$debug = 1;
	shift @arguments;
}

# check the system queue type: use SLURM if available, otherwise PBS
our $slurm_ok = (length(`which sbatch`) > 0);
our $pbs_ok = (length(`which qsub`) > 0);
print "SLURM detected.\n" if ($debug and $slurm_ok);
print "PBS also detected, but using SLURM only.\n" if ($debug and $slurm_ok and $pbs_ok);
print "PBS detected, no SLURM found..\n" if ($debug and !$slurm_ok and $pbs_ok);
print "No SLURM or PBS found. Only local jobs can be run.\n" if ($debug and !$slurm_ok and !$pbs_ok);
our $scheduler = ($slurm_ok ? "SLURM" : ($pbs_ok ? "PBS" : "none"));

# determine the current user name
our $user = getlogin () || getpwuid ($<);
print "Current user name: '$user'.\n" if $debug;

# determine the current user's home directory
#our $home = File::HomeDir -> my_home;
our $home = `echo \${HOME}`;
$home =~ s/\s*$//;
print "Home directory: '$home'.\n" if $debug;

# determine the zip directory
our $zipdir = "${home}/zip";
$zipdir = $home unless -d $zipdir;
print "Zip directory: '$zipdir'.\n" if $debug;

# determine the current directory
our $curdir = getcwd ();
print "Current directory name: '$curdir'.\n" if $debug;

# determine the core name of the computations
our $name = "comp";
$name = $1 if $curdir =~ m/[\/\\]([^\/\\]+)$/;
print "Computation name: '$name'.\n" if $debug;

# determine the number of computations
our $number = "0";
$number = $1 if $name =~ m/(\d+)[^\d]*$/;
print "Computation number: '$number'.\n" if $debug;

# assign a port for the communication
our $port = 7100;
$port = 7300 + $number if (($number > 0) && ($number < 100));
$port = 8000 + $number if (($number >= 100) && ($number < 1000));
print "Coordinator's port number: '$port'.\n" if $debug;

# determine the name of this computer
my $fullhost = hostname;
$fullhost =~ m/^([^\.]+)\.?(.*)/;
our $host = $1;
#our $domain = $2;

# make an attempt to determine on which host the coordinator is running
if ($slurm_ok or $pbs_ok)
{
	my $jobid;
	my @qstat = ($slurm_ok ? `squeue --user=${user} --format="%.18i %.30j %.8u %.2t %.10M %.6D %R"` : `qstat`);
	foreach (@qstat)
	{
		# squeue sample line: 25164078     batch MW.30-.7   osmark  R    1:51:36      1 gv1336
		# qstat sample line: 25164477            l1_+pa14         michal11        00:00:55 R batch  
		next unless m/^\s*(\d+).*COR\_${name}\s+${user}.*\s(\S+)\s*$/;
		$jobid = $1;
		$host = $2 if $slurm_ok;
		print "* jobid = $jobid\n" if $debug;
		last;
	}
	if (defined ($jobid) and !$slurm_ok and $pbs_ok)
	{
		my @qstat = `qstat -f $jobid`;
		foreach (@qstat)
		{
			next unless m/exec\_host\s*=\s*(\S+)/;
			$host = $1;
			$host =~ s/\/.*$//;
			print "* exec_host = '$host'.\n" if $debug;
			last;
		}
	}
}

print "Coordinator's host name: '$host'.\n" if $debug;
#print "Domain name: '$domain'.\n" if $debug;

# determine the coordinator's host:port
our $coord = "${host}:${port}";
#print "Coordinator: ${host}:${port}.\n" if $debug;

# set the working directory name prefix
our $dirprefix = "${home}/";
$dirprefix = "${home}/data/" if -d "${home}/data";
$dirprefix = "${home}/work/" if -d "${home}/work";
print "Work dir prefix: '$dirprefix'.\n" if $debug;

# set the directory base name
our $dirbase = $dirprefix . $name;
print "Directory base: '$dirbase'.\n" if $debug;

# set the names of all the directories in question
our $datdir = "${dirbase}.dat";
print "Data directory: '$datdir'.\n" if $debug;
our $cubdir = "${dirbase}.cub";
our $sharedir = "${dirbase}.ind";
our $pngdir = "${dirbase}.png";
our $odedir = "${dirbase}.ode";
our $decdir = "${dirbase}.dec";
our $cmgdir = "${dirbase}.cmg";
our $rundir = "${dirbase}.run";
our $postprocdir = "${dirbase}.pos";

# prepare a hash of the parameters defined above
my %param;
foreach my $key (keys %desc)
{
	die "Undefined $key." unless defined $$key;
	$param{$key} = $$key if defined $$key;
}

# output a space separated list of the requested data
my @results;
foreach my $arg (@arguments)
{
	push (@results, $param{$arg}) if defined $param{$arg};
}
print join (' ', @results);

